@@ -6,16 +6,11 @@ use crate::model;
 use crate::time;
 
 use super::evaluate;
+use super::evaluate::Scope;
 use super::*;
 
-pub type ActionLabelIndex = usize;
-
 pub type LocationIndex = usize;
 
-pub type EdgeIndex = usize;
-
-pub type DestinationIndex = usize;
-
 pub type GlobalEnvironment<'s> = evaluate::Environment<'s, 2>;
 pub type EdgeEnvironment<'s> = evaluate::Environment<'s, 3>;
 
@@ -24,19 +19,19 @@ pub struct CompiledAssignment {
     pub value: evaluate::CompiledExpression<3>,
 }
 
-pub struct CompiledAutomaton<Z: time::TimeType> {
+pub struct CompiledAutomaton<Z: time::Time> {
     pub reference: model::AutomatonReference,
     pub locations: Vec<CompiledLocation<Z>>,
 }
 
-pub struct CompiledLocation<Z: time::TimeType> {
+pub struct CompiledLocation<Z: time::Time> {
     pub reference: model::LocationReference,
     pub invariant: Vec<CompiledClockConstraint<Z>>,
     pub internal_edges: Vec<CompiledEdge<Z>>,
     pub visible_edges: Vec<Vec<CompiledVisibleEdge<Z>>>,
 }
 
-impl<Z: time::TimeType> CompiledLocation<Z> {
+impl<Z: time::Time> CompiledLocation<Z> {
     fn new(
         network: &model::Network,
         global_scope: &evaluate::Scope<2>,
@@ -109,18 +104,26 @@ impl<Z: time::TimeType> CompiledLocation<Z> {
     }
 }
 
-pub struct CompiledEdge<Z: time::TimeType> {
+pub struct CompiledEdge<Z: time::Time> {
     pub reference: model::EdgeReference,
     pub guard: CompiledGuard<Z>,
     pub destinations: Vec<CompiledDestination<Z>>,
+    pub observations: Vec<CompiledObservation>,
+    pub numeric_reference: (usize, usize),
 }
 
-pub struct CompiledVisibleEdge<Z: time::TimeType> {
+pub struct CompiledObservation {
+    pub label: LabelIndex,
+    pub arguments: Vec<evaluate::CompiledExpression<3>>,
+    pub probability: evaluate::CompiledExpression<3>,
+}
+
+pub struct CompiledVisibleEdge<Z: time::Time> {
     pub base: CompiledEdge<Z>,
     pub write_arguments: Box<[(usize, evaluate::CompiledExpression<2>)]>,
 }
 
-impl<Z: time::TimeType> CompiledEdge<Z> {
+impl<Z: time::Time> CompiledEdge<Z> {
     pub fn new(
         network: &model::Network,
         time_type: &Z,
@@ -175,10 +178,29 @@ impl<Z: time::TimeType> CompiledEdge<Z> {
                     .collect(),
             })
             .collect();
+        let observations = edge
+            .observations
+            .iter()
+            .map(|observation| CompiledObservation {
+                label: network
+                    .declarations
+                    .action_labels
+                    .get_index_of(&observation.label)
+                    .unwrap(),
+                arguments: observation
+                    .arguments
+                    .iter()
+                    .map(|argument| edge_scope.compile(argument))
+                    .collect(),
+                probability: edge_scope.compile(&observation.probability),
+            })
+            .collect();
         CompiledEdge {
             reference,
             guard,
             destinations,
+            observations,
+            numeric_reference: (automaton_index, edge.number),
         }
     }
 
@@ -191,18 +213,18 @@ impl<Z: time::TimeType> CompiledEdge<Z> {
     }
 }
 
-pub struct CompiledGuard<Z: time::TimeType> {
+pub struct CompiledGuard<Z: time::Time> {
     pub boolean_condition: evaluate::CompiledExpression<2>,
     pub clock_constraints: Vec<CompiledClockConstraint<Z>>,
 }
 
-pub struct CompiledClockConstraint<T: time::TimeType> {
+pub struct CompiledClockConstraint<T: time::Time> {
     pub difference: T::CompiledDifference,
     pub is_strict: bool,
     pub bound: evaluate::CompiledExpression<2>,
 }
 
-impl<T: time::TimeType> CompiledClockConstraint<T> {
+impl<T: time::Time> CompiledClockConstraint<T> {
     pub fn compile(
         time_type: &T,
         constraint: &model::ClockConstraint,
@@ -224,7 +246,7 @@ impl<T: time::TimeType> CompiledClockConstraint<T> {
     }
 }
 
-pub struct CompiledDestination<Z: time::TimeType> {
+pub struct CompiledDestination<Z: time::Time> {
     pub automaton_index: usize,
     pub reference: model::DestinationReference,
     pub location: LocationIndex,
@@ -244,7 +266,7 @@ impl SyncVectorItem {
         self.slot_mapping[argument_index]
     }
 
-    pub fn compute_link_edges<'c, Z: time::TimeType>(
+    pub fn compute_link_edges<'c, Z: time::Time>(
         &'c self,
         global_env: &evaluate::Environment<2>,
         enabled_edges: &Box<[Box<[Box<[&'c CompiledVisibleEdge<Z>]>]>]>,
@@ -267,7 +289,7 @@ impl SyncVectorItem {
     }
 }
 
-pub struct LinkEdge<'c, Z: time::TimeType> {
+pub struct LinkEdge<'c, Z: time::Time> {
     pub compiled: &'c CompiledVisibleEdge<Z>,
     pub write_slots: Box<[(usize, model::Value)]>,
 }
@@ -286,15 +308,16 @@ pub enum CompiledLinkResult {
     },
 }
 
-pub struct CompiledNetwork<Z: time::TimeType> {
+pub struct CompiledNetwork<Z: time::Time> {
     pub zone_compiler: Z,
+    pub global_scope: Scope<2>,
     pub automata: Box<[CompiledAutomaton<Z>]>,
     pub links: Box<[CompiledLink]>,
     pub transient_values: Box<[evaluate::CompiledExpression<1>]>,
     pub assignment_groups: IndexSet<usize>,
 }
 
-impl<Z: time::TimeType> CompiledNetwork<Z> {
+impl<Z: time::Time> CompiledNetwork<Z> {
     pub fn new(network: &model::Network) -> Self {
         let zone_compiler = Z::new(network).unwrap();
         let global_scope = network.global_scope();
@@ -389,6 +412,7 @@ impl<Z: time::TimeType> CompiledNetwork<Z> {
             .collect();
         CompiledNetwork {
             zone_compiler,
+            global_scope,
             automata,
             links,
             transient_values,
@@ -396,13 +420,21 @@ impl<Z: time::TimeType> CompiledNetwork<Z> {
         }
     }
 
+    pub fn compile_global_expression(
+        &self,
+        expr: &model::Expression,
+    ) -> evaluate::CompiledExpression<2> {
+        self.global_scope.compile(expr)
+    }
+
     pub fn compute_transition<'c>(
         &self,
-        zone: &Z::Valuations,
+        state: &State<Z>,
         global_env: &GlobalEnvironment,
         link: &'c CompiledLink,
         link_edges: &[&LinkEdge<'c, Z>],
     ) -> Option<Transition<'c, Z>> {
+        let zone = &state.valuations;
         debug_assert_eq!(link.sync_vector.len(), link_edges.len());
         let mut slots = link.slots_template.clone();
         let mut valuations = zone.clone();
@@ -440,7 +472,7 @@ impl<Z: time::TimeType> CompiledNetwork<Z> {
             .into_iter()
             .collect::<Option<Box<[_]>>>()
             .map(|slots| {
-                let actions = link
+                let actions: Box<[Action]> = link
                     .sync_vector
                     .iter()
                     .map(|vector_item| {
@@ -454,25 +486,50 @@ impl<Z: time::TimeType> CompiledNetwork<Z> {
                         )
                     })
                     .collect();
+                let observations = link_edges
+                    .iter()
+                    .zip(actions.iter())
+                    .map(|(link_edge, action)| {
+                        let edge_env = state.edge_env(action.arguments());
+                        link_edge
+                            .compiled
+                            .base
+                            .observations
+                            .iter()
+                            .map(|observation| Observation {
+                                label: observation.label,
+                                arguments: observation
+                                    .arguments
+                                    .iter()
+                                    .map(|argument| argument.evaluate(&edge_env))
+                                    .collect(),
+                                probability: observation.probability.evaluate(&edge_env),
+                            })
+                            .collect::<Box<_>>()
+                    })
+                    .collect();
                 Transition {
                     edges: link_edges
                         .iter()
                         .map(|link_edge| &link_edge.compiled.base)
                         .collect(),
-                    valuations,
-                    actions,
-                    action: match &link.result {
-                        CompiledLinkResult::Silent => Action::Silent,
-                        CompiledLinkResult::Labeled {
-                            action_label,
-                            slot_mapping,
-                        } => Action::new(
-                            *action_label,
-                            slot_mapping
-                                .iter()
-                                .map(|slot_index| slots[*slot_index].clone())
-                                .collect(),
-                        ),
+                    bare: BareTransition {
+                        valuations,
+                        actions,
+                        action: match &link.result {
+                            CompiledLinkResult::Silent => Action::Silent,
+                            CompiledLinkResult::Labeled {
+                                action_label,
+                                slot_mapping,
+                            } => Action::new(
+                                *action_label,
+                                slot_mapping
+                                    .iter()
+                                    .map(|slot_index| slots[*slot_index].clone())
+                                    .collect(),
+                            ),
+                        },
+                        observations,
                     },
                 }
             })
