@@ -1,17 +1,22 @@
 //! Algorithms and data structures for state space exploration.
 
+use std::marker::PhantomData;
+
 use serde::{Deserialize, Serialize};
 
 use itertools::Itertools;
 
+use crate::{
+    model::{EdgeReference, LabelIndex, Value},
+    time::Time,
+};
+
 use super::model;
 use super::time;
 
-mod actions;
-mod compiled;
-mod evaluate;
-
-pub mod external;
+pub mod actions;
+pub mod compiled;
+pub mod evaluate;
 
 use compiled::*;
 
@@ -22,24 +27,33 @@ pub use actions::*;
 /// Every state keeps track of the locations the automata of the network are in, the
 /// values of the global variables, and a potentially infinite set of clock valuations
 /// using the respective [TimeType][time::TimeType].
-#[derive(Serialize, Deserialize, Clone, Hash, Eq, PartialEq, Debug)]
-pub struct State<V> {
+#[derive(Serialize, Deserialize, Clone, Eq, PartialEq, Debug)]
+pub struct State<T: time::Time> {
     locations: Box<[LocationIndex]>,
     global_store: Box<[model::Value]>,
     transient_store: Box<[model::Value]>,
-    valuations: V,
+    valuations: T::Valuations,
 }
 
-impl<V> State<V> {
+impl<T: time::Time> std::hash::Hash for State<T> {
+    fn hash<H: std::hash::Hasher>(&self, hasher: &mut H) {
+        self.locations.hash(hasher);
+        self.global_store.hash(hasher);
+        self.transient_store.hash(hasher);
+        self.valuations.hash(hasher);
+    }
+}
+
+impl<T: time::Time> State<T> {
     /// Returns the name of the location the automaton with the provided name is in.
     ///
     /// Panics in case the state has not been produced by the provided explorer or there
     /// is no automaton with the provided name in the automaton network.
-    pub fn get_location_of<'e, T: time::TimeType>(
+    pub fn get_location_of<'e>(
         &self,
         explorer: &'e Explorer<T>,
         automaton_name: &str,
-    ) -> &'e String {
+    ) -> Option<&'e String> {
         explorer
             .network
             .automata
@@ -59,25 +73,23 @@ impl<V> State<V> {
                             .map(|(location_name, _)| location_name)
                     })
             })
-            .expect("Invalid automaton name or explorer passed to `State::get_location_of`.")
     }
 
     /// Returns the value of the provided global variable.
     ///
     /// Panics in case the state has not been produced by the provided explorer or there
     /// is no global variable with the provided name in the automaton network.
-    pub fn get_global_value<T: time::TimeType>(
+    pub fn get_global_value(
         &self,
         explorer: &Explorer<T>,
         identifier: &str,
-    ) -> &model::Value {
+    ) -> Option<&model::Value> {
         explorer
             .network
             .declarations
             .global_variables
             .get_index_of(identifier)
             .and_then(|index| self.global_store.get(index))
-            .expect("Invalid variable name or explorer passed to `State::get_global_value`.")
     }
 
     /// Returns the value of the provided transient variable.
@@ -93,8 +105,13 @@ impl<V> State<V> {
             .expect("Invalid variable name or explorer passed to `State::get_transient_value`.")
     }
 
+    /// Evaluates a compiled expression on the state.
+    pub fn evaluate(&self, expr: &evaluate::CompiledExpression<2>) -> model::Value {
+        expr.evaluate(&self.global_env())
+    }
+
     /// Returns the clock valuations associated with the state.
-    pub fn valuations(&self) -> &V {
+    pub fn valuations(&self) -> &T::Valuations {
         &self.valuations
     }
 
@@ -108,10 +125,7 @@ impl<V> State<V> {
         evaluate::Environment::new([&self.global_store, &self.transient_store, edge_store])
     }
 
-    fn future<T: time::TimeType>(
-        state: State<T::Valuations>,
-        compiled_network: &CompiledNetwork<T>,
-    ) -> State<T::Valuations> {
+    fn future(state: State<T>, compiled_network: &CompiledNetwork<T>) -> State<T> {
         let env = evaluate::Environment::new([&state.global_store, &state.transient_store]);
 
         let mut valuations = compiled_network.zone_compiler.future(state.valuations);
@@ -140,18 +154,84 @@ impl<V> State<V> {
     }
 }
 
-/// A *transition* of an automaton network.
-pub struct Transition<'e, T: time::TimeType> {
-    pub(crate) edges: Box<[&'e CompiledEdge<T>]>,
+#[derive(Hash, Eq, PartialEq, Clone, Debug)]
+pub struct Observation {
+    pub label: LabelIndex,
+    pub arguments: Box<[Value]>,
+    pub probability: Value,
+}
+
+pub(crate) struct BareTransition<T: time::Time> {
     pub(crate) actions: Box<[Action]>,
     pub(crate) valuations: T::Valuations,
     pub(crate) action: Action,
+    pub(crate) observations: Box<[Box<[Observation]>]>,
 }
 
-impl<'e, T: time::TimeType> Transition<'e, T> {
+pub trait AnyTransition<T: time::Time> {
+    fn result_action(&self) -> &Action;
+
+    fn valuations(&self) -> &T::Valuations;
+}
+
+/// A *transition* of an automaton network.
+pub struct DetachedTransition<T: time::Time> {
+    pub(crate) edges: Vec<EdgeReference>,
+    pub(crate) bare: BareTransition<T>,
+}
+
+impl<T: time::Time> DetachedTransition<T> {
+    /// Attaches the transition to the lifetime of the explorer.
+    pub fn attach(self, explorer: &Explorer<T>) -> Transition<T> {
+        todo!()
+    }
+
+    pub fn edge_vector(&self) -> &[EdgeReference] {
+        &self.edges
+    }
+
+    pub fn action_vector(&self) -> &[Action] {
+        &self.bare.actions
+    }
+
+    /// Replaces the valuations of the transition.
+    pub fn set_valuations(&mut self, valuations: T::Valuations) {
+        self.bare.valuations = valuations;
+    }
+
+    pub fn valuations(&self) -> &T::Valuations {
+        &self.bare.valuations
+    }
+
+    pub fn result_action(&self) -> &Action {
+        &self.bare.action
+    }
+}
+
+/// A *transition* attached to an [Explorer].
+pub struct Transition<'e, T: time::Time> {
+    pub(crate) edges: Box<[&'e CompiledEdge<T>]>,
+    pub(crate) bare: BareTransition<T>,
+}
+
+impl<'e, T: time::Time> Transition<'e, T> {
+    /// Detaches the transition from the lifetime of the explorer.
+    pub fn detach(self) -> DetachedTransition<T> {
+        DetachedTransition {
+            edges: self
+                .edges
+                .into_iter()
+                .map(|edge| edge.reference.clone())
+                .collect(),
+            bare: self.bare,
+        }
+    }
+}
+
+impl<'e, T: time::Time> Transition<'e, T> {
     /// Returns a slice of actions the participating automata perform.
     pub fn local_actions(&self) -> &[Action] {
-        &self.actions
+        &self.bare.actions
     }
 
     /// Returns a vector of indices of the participating automata.
@@ -164,21 +244,28 @@ impl<'e, T: time::TimeType> Transition<'e, T> {
 
     /// Returns the clock valuations for which the transition is performed.
     pub fn valuations(&self) -> &T::Valuations {
-        &self.valuations
+        &self.bare.valuations
     }
 
     /// Returns the resulting action.
     pub fn result_action(&self) -> &Action {
-        &self.action
+        &self.bare.action
+    }
+
+    pub fn set_valuations(&mut self, valuations: T::Valuations) {
+        self.bare.valuations = valuations;
     }
 
     /// Replaces the valuations of the transition.
     pub fn replace_valuations(self, valuations: T::Valuations) -> Self {
         Transition {
             edges: self.edges,
-            actions: self.actions,
-            valuations,
-            action: self.action,
+            bare: BareTransition {
+                actions: self.bare.actions,
+                valuations,
+                action: self.bare.action,
+                observations: self.bare.observations,
+            },
         }
     }
 
@@ -188,15 +275,26 @@ impl<'e, T: time::TimeType> Transition<'e, T> {
             .map(|edge| edge.reference.clone())
             .collect()
     }
+
+    pub fn observations(&self) -> &[Box<[Observation]>] {
+        &self.bare.observations
+    }
+
+    pub fn numeric_reference_vector(&self) -> Vec<(usize, usize)> {
+        self.edges
+            .iter()
+            .map(|edge| edge.numeric_reference.clone())
+            .collect()
+    }
 }
 
 /// A *destination* of a transition.
-pub struct Destination<'c, T: time::TimeType> {
+pub struct Destination<'c, T: time::Time> {
     pub(crate) probability: f64,
     pub(crate) destinations: Box<[&'c CompiledDestination<T>]>,
 }
 
-impl<'c, T: time::TimeType> Destination<'c, T> {
+impl<'c, T: time::Time> Destination<'c, T> {
     /// Returns the probability of the destination.
     pub fn probability(&self) -> f64 {
         self.probability
@@ -204,12 +302,12 @@ impl<'c, T: time::TimeType> Destination<'c, T> {
 }
 
 /// A state space explorer for a particular automaton network.
-pub struct Explorer<T: time::TimeType> {
+pub struct Explorer<T: time::Time> {
     pub network: model::Network,
-    pub(crate) compiled_network: CompiledNetwork<T>,
+    pub compiled_network: CompiledNetwork<T>,
 }
 
-impl<T: time::TimeType> Explorer<T> {
+impl<T: time::Time> Explorer<T> {
     /// Constructs a new state space explorer from the provided network.
     pub fn new(network: model::Network) -> Self {
         let compiled = CompiledNetwork::new(&network);
@@ -220,7 +318,7 @@ impl<T: time::TimeType> Explorer<T> {
     }
 
     /// Returns a vector of initial states of the network.
-    pub fn initial_states(&self) -> Vec<State<T::Valuations>> {
+    pub fn initial_states(&self) -> Vec<State<T>> {
         let global_scope = self.network.global_scope();
         self.network
             .initial_states
@@ -267,7 +365,7 @@ impl<T: time::TimeType> Explorer<T> {
                             .collect(),
                     )
                     .unwrap();
-                State::<T::Valuations>::future(
+                State::<T>::future(
                     State {
                         locations,
                         global_store,
@@ -281,7 +379,7 @@ impl<T: time::TimeType> Explorer<T> {
     }
 
     /// Returns a vector of outgoing transitions of the given state.
-    pub fn transitions<'c>(&'c self, state: &State<T::Valuations>) -> Vec<Transition<'c, T>> {
+    pub fn transitions<'c>(&'c self, state: &State<T>) -> Vec<Transition<'c, T>> {
         let global_env = state.global_env();
         let enabled_edges = self
             .compiled_network
@@ -331,9 +429,19 @@ impl<T: time::TimeType> Explorer<T> {
                         } else {
                             Some(Transition {
                                 edges: Box::new([edge]),
-                                actions: Box::new([Action::Silent]),
-                                valuations,
-                                action: Action::Silent,
+                                bare: BareTransition {
+                                    actions: Box::new([Action::Silent]),
+                                    valuations,
+                                    action: Action::Silent,
+                                    observations: edge
+                                        .observations
+                                        .iter()
+                                        .map(|observation| {
+                                            let edge_env = state.edge_env(&[]);
+                                            todo!("observations on silent edges are not supported yet");
+                                        })
+                                        .collect(),
+                                },
                             })
                         }
                     }
@@ -355,7 +463,7 @@ impl<T: time::TimeType> Explorer<T> {
                             .multi_cartesian_product()
                             .filter_map(|edges| {
                                 self.compiled_network.compute_transition(
-                                    &state.valuations,
+                                    &state,
                                     &global_env,
                                     link,
                                     &edges,
@@ -373,7 +481,7 @@ impl<T: time::TimeType> Explorer<T> {
     /// Panics if the transition has not been generated from the provided state.
     pub fn destinations<'c>(
         &'c self,
-        state: &State<T::Valuations>,
+        state: &State<T>,
         transition: &Transition<'c, T>,
     ) -> Vec<Destination<'c, T>> {
         transition
@@ -382,9 +490,12 @@ impl<T: time::TimeType> Explorer<T> {
             .map(|edge| edge.destinations.iter())
             .multi_cartesian_product()
             .map(|destinations| Destination {
-                probability: transition.actions.iter().zip(destinations.iter()).fold(
-                    1.0,
-                    |probability, (action, destination)| {
+                probability: transition
+                    .bare
+                    .actions
+                    .iter()
+                    .zip(destinations.iter())
+                    .fold(1.0, |probability, (action, destination)| {
                         let edge_env = state.edge_env(action.arguments());
                         probability
                             * destination
@@ -392,8 +503,7 @@ impl<T: time::TimeType> Explorer<T> {
                                 .evaluate(&edge_env)
                                 .unwrap_float64()
                                 .into_inner()
-                    },
-                ),
+                    }),
                 destinations: destinations.into(),
             })
             .collect()
@@ -404,10 +514,10 @@ impl<T: time::TimeType> Explorer<T> {
     /// Panics if the destination has not been generated from the provided state and transition.
     pub fn successor<'c>(
         &'c self,
-        state: &State<T::Valuations>,
+        state: &State<T>,
         transition: &Transition<'c, T>,
         destination: &Destination<T>,
-    ) -> State<T::Valuations> {
+    ) -> State<T> {
         let mut targets = vec![
             model::Value::Vector(state.global_store.clone().into()),
             model::Value::Vector(state.transient_store.clone().into()),
@@ -415,6 +525,7 @@ impl<T: time::TimeType> Explorer<T> {
         for index in 0..self.compiled_network.assignment_groups.len() {
             let global_store = targets[0].clone();
             for (action, automaton_destination) in transition
+                .bare
                 .actions
                 .iter()
                 .zip(destination.destinations.iter())
@@ -446,7 +557,7 @@ impl<T: time::TimeType> Explorer<T> {
             .compiled_network
             .compute_transient_values(&targets[0].unwrap_vector());
 
-        let state = State::<T::Valuations>::future(
+        let successor = State::<T>::future(
             State {
                 locations,
                 global_store: targets[0].unwrap_vector().clone().into(),
@@ -457,32 +568,29 @@ impl<T: time::TimeType> Explorer<T> {
         );
 
         // The truth of the guard should imply the truth of the invariants.
-        assert!(!self
+        if self
             .compiled_network
             .zone_compiler
-            .is_empty(&state.valuations));
+            .is_empty(&successor.valuations)
+        {
+            panic!("the truth of the guards should imply the truth of the invariants");
+        }
 
-        state
+        successor
     }
 
-    pub fn externalize_transition(&self, transition: &Transition<T>) -> external::Transition<T> {
-        external::Transition {
-            edge_vector: transition
-                .edges
-                .iter()
-                .map(|edge| edge.reference.clone())
-                .collect(),
-            action_vector: transition.actions.clone(),
-            action: transition.action.clone(),
-            valuations: self
-                .compiled_network
-                .zone_compiler
-                .externalize(transition.valuations.clone()),
-        }
+    pub fn get_time_type(&self) -> &T {
+        &self.compiled_network.zone_compiler
     }
 }
 
-/// A specialization of [Explorer] for MDPs using [NoClocks][time::NoClocks].
-///
-/// MDPs do not have any real-valued clocks.
-pub type MDPExplorer = Explorer<time::NoClocks>;
+/// Type definitions for *Markov Decision Processes* (MDPs).
+pub mod mdp {
+    /// The [Time][super::time::Time] type to use for MDPs.
+    pub type Time = super::time::NoClocks;
+
+    /// The [Explorer][super::Explorer] type to use for MDPs.
+    pub type Explorer = super::Explorer<Time>;
+    /// The [State][super::State] type to use for MDPs.
+    pub type State = super::State<Time>;
+}
